/**
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "Features/FeatureExtractor.h"

#include "ImageMathUtil.h"

#include <opencv/cv.h>

namespace MediaVision {
namespace Image {

const cv::Size FeatureExtractor::__MIN_SIZE = cv::Size(5, 5);

FeatureExtractor::FeatureExtractor() :
		__kpType(KT_INVALID),
		__detector(),
		__descType(DT_INVALID),
		__extractor(),
		__computeRecognitionRate(NULL)
{
}

void FeatureExtractor::setFeatureDetector(
		const cv::Ptr<cv::FeatureDetector> detector,
		KeypointType keypointType)
{
	__detector = detector;
	__kpType = keypointType;
}

void FeatureExtractor::setDescriptorExtractor(
		cv::Ptr<cv::DescriptorExtractor> extractor,
		DescriptorType descriptorType)
{
	__extractor = extractor;
	__descType = descriptorType;
}

void FeatureExtractor::setRecognitionRateMetric(
		float (*computeRecognitionRate)(
				const cv::Mat&,
				const std::vector<cv::KeyPoint>&))
{
	__computeRecognitionRate = computeRecognitionRate;
}

bool FeatureExtractor::extract(
		const cv::Mat& image,
		FeaturePack& result,
		const std::vector<cv::Point2f>& roi)
{
	if (__detector.empty() || __extractor.empty())
		return false;

	cv::Rect boundingBox;

	if (roi.empty()) {
		boundingBox.x = 0;
		boundingBox.y = 0;
		boundingBox.width = image.cols;
		boundingBox.height = image.rows;
	} else {
		if (roi.size() < 3)
			return false;

		boundingBox = cv::boundingRect(roi);
		catRect(boundingBox, image.size());
	}

	if (boundingBox.width < __MIN_SIZE.width || boundingBox.height < __MIN_SIZE.height)
		return false;

	result.__objectKeypoints.clear();

	std::vector<cv::KeyPoint> keypoints;

	__detector->detect(
			image(boundingBox),
			keypoints);

	result.__objectKeypoints = keypoints;

	if (!roi.empty()) {
		const size_t numberOfKeypoints = keypoints.size();
		result.__objectKeypoints.resize(numberOfKeypoints);
		for (size_t i = 0; i < numberOfKeypoints; ++i) {
			result.__objectKeypoints[i].pt.x += boundingBox.x;
			result.__objectKeypoints[i].pt.y += boundingBox.y;
		}
	}

	if (!roi.empty()) {
		/* TODO: Ecode roi to reduce the boundary effect. Provide new parameter
				for this action cause roi is a bounding contour for the object. */

		for (size_t i = 0; i < result.__objectKeypoints.size(); ++i) {
			if (!checkAccessory(result.__objectKeypoints[i].pt, roi)) {
				result.__objectKeypoints.erase(result.__objectKeypoints.begin() + i);
				--i;
			}
		}
	}

	 __extractor->compute(
			image,
			result.__objectKeypoints,
			result.__objectDescriptors);

	if (NULL != __computeRecognitionRate) {
		result.__recognitionRate = __computeRecognitionRate(
				image(boundingBox),
				keypoints);
	} else {
		/* Default recognition rate metric */
		if (result.__objectKeypoints.size() < MinimumNumberOfFeatures)
			result.__recognitionRate = 0.f;
		else
			result.__recognitionRate = 0.5f;
	}

	result.__keypointsType = __kpType;
	result.__descriptorsType = __descType;

	return true;
}

} /* Image */
} /* MediaVision */
