/**
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ImageMathUtil.h"

#include "Tracking/ImageContourStabilizator.h"

#include "mv_private.h"

namespace MediaVision {
namespace Image {

ImageContourStabilizator::ImageContourStabilizator() :
		__movingHistory(),
		__priorities()
{
	reset();
}

void ImageContourStabilizator::reset(void)
{
	__tolerantShift = 0.0f;
	__tolerantShiftExtra = 0.0f;
	__isPrepared = false;
	__tempContourIndex = -1;
	__currentHistoryAmount = 0;
	__historyAmount = 0;
	__movingHistory.clear();
}

ImageContourStabilizator::StabilizationError ImageContourStabilizator::stabilize(
		std::vector<cv::Point2f>& contour,
		const StabilizationParams& params)
{
	if (!updateSettings(params)) {
		LOGW("Not stabilized. Invalid settings.");

		return InvalidSettings;
	}

	/* current implementation stabilizes quadrangles only */
	if (contour.size() != NumberOfQuadrangleCorners) {
		LOGW("Not stabilized. Unsupported contour type.");

		return UnsupportedContourType;
	}

	__currentCornersSpeed.resize(contour.size(), 0);

	if (__lastStabilizedContour.empty())
		__lastStabilizedContour = contour;

	std::vector<cv::Point2f> stabilizedState;

	/* history amount < 2 it's no sense */
	if (__historyAmount >= 2) {
		/* first sample */
		if (__tempContourIndex == -1) {
			__movingHistory.push_back(contour);
			__movingHistory.push_back(contour);

			__tempContourIndex = 1;
			__currentHistoryAmount = 2;

			LOGI("Not stabilized. Too small moving history. (the first one)");

			return TooShortMovingHistory;
		}

		/* too short moving history */
		if (__currentHistoryAmount < __historyAmount) {
			++__currentHistoryAmount;
			++__tempContourIndex;
			__movingHistory.push_back(contour);

			LOGI("Not stabilized. Too short moving history.");

			return TooShortMovingHistory;
		}

		/* saving into moving history */
		__movingHistory.pop_front();
		__movingHistory.push_back(contour);

		if (!__isPrepared) {
			__lastStabilizedContour = __movingHistory[__historyAmount - 2];

			LOGI("Not stabilized. Too small moving history. (the last one)");

			__isPrepared = true;
		}

		/* stabilization */
		stabilizedState = computeStabilizedQuadrangleContour();

		if (stabilizedState.empty())
			stabilizedState = __lastStabilizedContour;
	} else {
		stabilizedState = __lastStabilizedContour;
	}

	const float tolerantShift = getQuadrangleArea(contour.data()) *
				__tolerantShift + __tolerantShiftExtra;

	const size_t contourSize = stabilizedState.size();
	for (size_t i = 0u; i < contourSize; ++i) {
		if (fabs(getDistance(stabilizedState[i], contour[i])) > tolerantShift) {
			const float dirX = __lastStabilizedContour[i].x - contour[i].x;
			const float dirY = __lastStabilizedContour[i].y - contour[i].y;

			const float speedX = dirX * __speeds[__currentCornersSpeed[i]];
			const float speedY = dirY * __speeds[__currentCornersSpeed[i]];

			/* final moving */
			__lastStabilizedContour[i].x -= speedX;
			__lastStabilizedContour[i].y -= speedY;

			if (__currentCornersSpeed[i] < __speeds.size() - 1)
				++__currentCornersSpeed[i];
		} else {
			__currentCornersSpeed[i] = 0;
		}
	}

	/* __lastStabilizedContour = stabilizedState; */
	contour = __lastStabilizedContour;

	LOGI("Contour successfully stabilized.");

	return Successfully;
}

bool ImageContourStabilizator::updateSettings(const StabilizationParams& params)
{
	if (params.mHistoryAmount < 1)
		return false;

	__tolerantShift = (float)params.mTolerantShift;
	__tolerantShiftExtra = (float)params.mTolerantShiftExtra;

	if (__historyAmount != (size_t)params.mHistoryAmount) {
		__historyAmount = (size_t)params.mHistoryAmount;

		__priorities.resize(__historyAmount);

		/* calculation of priorities for positions in the moving history */
		for (size_t i = 0u; i < __historyAmount; ++i) {
			/* linear dependence on the elapsed time */
			__priorities[i] = ((i + 1) * 2.0f) /
						((__historyAmount + 1) * __historyAmount);
		}
	}

	while (__historyAmount > (size_t)params.mHistoryAmount) {
		__movingHistory.pop_front();
		--__historyAmount;
	}

	if ((size_t)params.mHistoryAmount > __historyAmount) {
		/* TODO: save current moving history */

		__tempContourIndex = -1;
		__historyAmount = (size_t)params.mHistoryAmount;
		__movingHistory.clear();
	}

	bool speedIsValid = false;
	if (__speeds.size() > 1) {
		const static float __EPSILON = 0.0001f;
		if (fabs(__speeds[0] - params.mStabilizationSpeed) < __EPSILON &&
				fabs((__speeds[1] - __speeds[0]) -
				params.mStabilizationAcceleration) < __EPSILON) {
			speedIsValid = true;
		}
	}

	if (!speedIsValid) {
		__speeds.clear();

		int speedsSize = (int)((1 - params.mStabilizationSpeed) /
					params.mStabilizationAcceleration) + 1;

		if (speedsSize < 1) {
			__speeds.push_back(1.0f);
		} else {
			static const int MAX_SPEED_SIZE = 25;

			if (speedsSize > MAX_SPEED_SIZE)
				speedsSize = MAX_SPEED_SIZE;

			float speed = std::max(0.f,
					std::min((float)params.mStabilizationSpeed, 1.0f));

			for (int i = 0; i < speedsSize; ++i) {
				__speeds.push_back(speed);
				speed += params.mStabilizationAcceleration;
			}
		}
	}

	return true;
}

std::vector<cv::Point2f> ImageContourStabilizator::computeStabilizedQuadrangleContour(void)
{
	/* final contour */
	std::vector<cv::Point2f> stabilizedState(
				NumberOfQuadrangleCorners, cv::Point2f(0.f, 0.f));

	/* calculation the direction of contour corners to a new location */
	std::vector<cv::Point2f> directions(
				NumberOfQuadrangleCorners, cv::Point2f(0.f, 0.f));

	/* computing expected directions and outliers searching */
	bool expressiveTime = false;
	float summPriorityWithoutToLastPos[NumberOfQuadrangleCorners];
	float priorityToLastPos[NumberOfQuadrangleCorners];
	std::vector<cv::Point2f> directionsToLastPos(NumberOfQuadrangleCorners);
	for (size_t j = 0u; j < NumberOfQuadrangleCorners; ++j) {
		/* calculation the moving directions and computing average direction */
		std::vector<cv::Point2f> trackDirections(__historyAmount - 1);
		cv::Point2f averageDirections(0.f, 0.f);

		for (size_t i = 0u; i < __historyAmount - 1; ++i) {
			averageDirections.x += (trackDirections[i].x =
					__movingHistory[i+1][j].x - __movingHistory[i][j].x) /
					(__historyAmount - 1);

			averageDirections.y += (trackDirections[i].y =
					__movingHistory[i+1][j].y - __movingHistory[i][j].y) /
					(__historyAmount - 1);
		}

		/* calculation a deviations and select outlier */
		std::vector<float> directionDistances(__historyAmount - 1);
		float maxDistance = 0.f, prevMaxDistance = 0.f;
		int idxWithMaxDistance = 0;
		int numExpressiveDirection = -1;
		for (size_t i = 0u; i < __historyAmount - 1; ++i) {
			directionDistances[i] = getDistance(
					trackDirections[i],
					averageDirections);

			if (directionDistances[i] > prevMaxDistance) {
				if (directionDistances[i] > maxDistance) {
					prevMaxDistance = maxDistance;
					maxDistance = directionDistances[i];
					idxWithMaxDistance = i;
				} else {
					prevMaxDistance = directionDistances[i];
				}
			}
		}

		/* check outlier */
		if (0.6f * maxDistance > prevMaxDistance) {
			LOGI("Outlier is detected.");

			numExpressiveDirection = idxWithMaxDistance;
		}

		/* final direction computing */
		float summPriority = 0.f;
		for (size_t i = 0u; i < __historyAmount - 1; ++i) {
			if ((int)i != numExpressiveDirection) {
				directions[j].x += trackDirections[i].x * __priorities[i];
				directions[j].y += trackDirections[i].y * __priorities[i];
				summPriority += __priorities[i];
			}
		}

		if (numExpressiveDirection == (int)(__historyAmount - 1))
			expressiveTime = true;

		summPriorityWithoutToLastPos[j] = summPriority;
		priorityToLastPos[j] = __priorities[__historyAmount - 1];

		directions[j].x -= directionsToLastPos[j].x =
					(__lastStabilizedContour[j].x -
					__movingHistory[__historyAmount - 1][j].x) *
					priorityToLastPos[j];

		directions[j].y -= directionsToLastPos[j].y =
					(__lastStabilizedContour[j].y -
					__movingHistory[__historyAmount - 1][j].y) *
					priorityToLastPos[j];

		summPriority += priorityToLastPos[j];

		directions[j].x /= summPriority;
		directions[j].y /= summPriority;
	}

	/* final corners computing */
	for (size_t j = 0u; j < NumberOfQuadrangleCorners; ++j) {
		if (expressiveTime) {
			directions[j].x *= (summPriorityWithoutToLastPos[j] +
						priorityToLastPos[j]);
			directions[j].x -= directionsToLastPos[j].x;
			directions[j].x /= summPriorityWithoutToLastPos[j];

			directions[j].y *= (summPriorityWithoutToLastPos[j] +
						priorityToLastPos[j]);
			directions[j].y -= directionsToLastPos[j].y;
			directions[j].y /= summPriorityWithoutToLastPos[j];
		}

		stabilizedState[j].x = __lastStabilizedContour[j].x + directions[j].x;
		stabilizedState[j].y = __lastStabilizedContour[j].y + directions[j].y;
	}

	return stabilizedState;
}

} /* Image */
} /* MediaVision */
